@@ -38,7 +38,6 @@
 
 #include "soundmgr.hxx"
 #include "readwav.hxx"
-#include "soundmgr_openal_private.hxx"
 #include "sample_group.hxx"
 
 #include <simgear/sg_inlines.h>
@@ -46,6 +45,17 @@
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 
+#if defined(__APPLE__)
+# include <OpenAL/al.h>
+# include <OpenAL/alc.h>
+#elif defined(OPENALSDK)
+# include <al.h>
+# include <alc.h>
+#else
+# include <AL/al.h>
+# include <AL/alc.h>
+#endif
+
 using std::vector;
 
 
@@ -64,19 +74,29 @@ using std::vector;
 # define AL_UNPACK_BLOCK_ALIGNMENT_SOFT	0x200C
 #endif
 
+
+struct refUint {
+    unsigned int refctr;
+    ALuint id;
+
+    refUint() { refctr = 0; id = (ALuint)-1; };
+    refUint(ALuint i) { refctr = 1; id = i; };
+    ~refUint() {};
+};
+
+using buffer_map = std::map < std::string, refUint >;
+using sample_group_map = std::map < std::string, SGSharedPtr<SGSampleGroup> >;
+
+inline bool isNaN(float *v) {
+   return (SGMisc<float>::isNaN(v[0]) || SGMisc<float>::isNaN(v[1]) || SGMisc<float>::isNaN(v[2]));
+}
+
+
 class SGSoundMgr::SoundManagerPrivate
 {
 public:
-    SoundManagerPrivate() :
-        _device(NULL),
-        _context(NULL),
-        _absolute_pos(SGVec3d::zeros()),
-        _base_pos(SGVec3d::zeros()),
-        _orientation(SGQuatd::zeros())
-    {
-        
-        
-    }
+    SoundManagerPrivate() = default;
+    ~SoundManagerPrivate() = default;
     
     void init()
     {
@@ -109,18 +129,20 @@ public:
         _absolute_pos = _base_pos;
     }
         
-    ALCdevice *_device;
-    ALCcontext *_context;
+    ALCdevice *_device = nullptr;
+    ALCcontext *_context = nullptr;
     
     std::vector<ALuint> _free_sources;
     std::vector<ALuint> _sources_in_use;
     
-    SGVec3d _absolute_pos;
-    SGVec3d _base_pos;
-    SGQuatd _orientation;
+    SGVec3d _absolute_pos = SGVec3d::zeros();
+    SGVec3d _base_pos = SGVec3d::zeros();
+    SGQuatd _orientation = SGQuatd::zeros();
     // Orientation of the listener. 
     // first 3 elements are "at" vector, second 3 are "up" vector
     ALfloat _at_up_vec[6];
+
+    bool _bad_doppler = false;
     
     sample_group_map _sample_groups;
     buffer_map _buffers;
@@ -132,15 +154,7 @@ public:
 //
 
 // constructor
-SGSoundMgr::SGSoundMgr() :
-    _active(false),
-    _changed(true),
-    _volume(0.0),
-    _velocity(SGVec3d::zeros()),
-    _bad_doppler(false),
-    _renderer("unknown"),
-    _vendor("unknown")
-{
+SGSoundMgr::SGSoundMgr() {
     d.reset(new SoundManagerPrivate);
     d->_base_pos = SGVec3d::fromGeod(_geod_pos);
 
@@ -173,10 +187,10 @@ void SGSoundMgr::init()
     d->_sources_in_use.clear();
     d->_sources_in_use.reserve( MAX_SOURCES );
 
-    ALCdevice *device = NULL;
+    ALCdevice *device = nullptr;
     const char* devname = _device_name.c_str();
     if (_device_name == "")
-        devname = NULL; // use default device
+        devname = nullptr; // use default device
     else
     {
         // try non-default device
@@ -184,13 +198,13 @@ void SGSoundMgr::init()
     }
 
     if ((!devname)||(testForError(device, "Audio device not available, trying default.")) ) {
-        device = alcOpenDevice(NULL);
+        device = alcOpenDevice(nullptr);
         if (testForError(device, "Default audio device not available.") ) {
            return;
         }
     }
 
-    ALCcontext *context = alcCreateContext(device, NULL);
+    ALCcontext *context = alcCreateContext(device, nullptr);
     testForALCError("context creation.");
     if ( testForError(context, "Unable to create a valid context.") ) {
         alcCloseDevice (device);
@@ -204,7 +218,7 @@ void SGSoundMgr::init()
         return;
     }
 
-    if (d->_context != NULL)
+    if (d->_context != nullptr)
     {
         SG_LOG(SG_SOUND, SG_ALERT, "context is already assigned");
     }
@@ -218,8 +232,8 @@ void SGSoundMgr::init()
     alListenerfv( AL_POSITION, SGVec3f::zeros().data() );
     alListenerfv( AL_VELOCITY, SGVec3f::zeros().data() );
 
-    alDopplerFactor(1.0);
-    alDopplerVelocity(340.3);   // speed of sound in meters per second.
+    alDopplerFactor(1.0f);
+    alDopplerVelocity(_sound_velocity);
 
     // gain = AL_REFERENCE_DISTANCE / (AL_REFERENCE_DISTANCE +
     //        AL_ROLLOFF_FACTOR * (distance - AL_REFERENCE_DISTANCE));
@@ -248,9 +262,11 @@ void SGSoundMgr::init()
     _renderer = (const char *)alGetString(AL_RENDERER);
 
     if (_vendor == "Creative Labs Inc.") {
-       _bad_doppler = true;
+       alDopplerFactor(100.0f);
+       d->_bad_doppler = true;
     } else if (_vendor == "OpenAL Community" && _renderer == "OpenAL Soft") {
-       _bad_doppler = true;
+       alDopplerFactor(100.0f);
+       d->_bad_doppler = true;
     }
 
     if (d->_free_sources.empty()) {
@@ -281,10 +297,8 @@ void SGSoundMgr::activate()
     if ( is_working() ) {
         _active = true;
                 
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->activate();
+        for ( auto current : d->_sample_groups ) {
+            current.second->activate();
         }
     }
 #endif
@@ -295,10 +309,8 @@ void SGSoundMgr::stop()
 {
 #ifdef ENABLE_SOUND
     // first stop all sample groups
-    for ( auto current = d->_sample_groups.begin();
-               current != d->_sample_groups.end(); ++current ) {
-        SGSampleGroup *sgrp = current->second;
-        sgrp->stop();
+    for ( auto current : d->_sample_groups ) {
+        current.second->stop();
     }
 
     // clear all OpenAL sources
@@ -309,9 +321,8 @@ void SGSoundMgr::stop()
     d->_free_sources.clear();
 
     // clear any OpenAL buffers before shutting down
-    for ( auto current = d->_buffers.begin();
-               current != d->_buffers.begin(); ++current ) {
-        refUint ref = current->second;
+    for ( auto current : d->_buffers ) {
+        refUint ref = current.second;
         ALuint buffer = ref.id;
         alDeleteBuffers(1, &buffer);
         testForError("SGSoundMgr::stop: delete buffers");
@@ -324,8 +335,8 @@ void SGSoundMgr::stop()
         _active = false;
         alcDestroyContext(d->_context);
         alcCloseDevice(d->_device);
-        d->_context = NULL;
-        d->_device = NULL;
+        d->_context = nullptr;
+        d->_device = nullptr;
 
         _renderer = "unknown";
         _vendor = "unknown";
@@ -337,10 +348,8 @@ void SGSoundMgr::suspend()
 {
 #ifdef ENABLE_SOUND
     if (is_working()) {
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->stop();
+        for ( auto current : d->_sample_groups ) {
+            current.second->suspend();
         }
         _active = false;
     }
@@ -351,10 +360,8 @@ void SGSoundMgr::resume()
 {
 #ifdef ENABLE_SOUND
     if (is_working()) {
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->resume();
+        for ( auto current : d->_sample_groups ) {
+            current.second->resume();
         }
         _active = true;
     }
@@ -372,10 +379,8 @@ void SGSoundMgr::update( double dt )
             d->update_pos_and_orientation();
         }
 
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->update(dt);
+        for ( auto current : d->_sample_groups ) {
+            current.second->update(dt);
         }
 
         if (_changed) {
@@ -392,14 +397,20 @@ if (isNaN(toVec3f(_velocity).data())) printf("NaN in listener velocity\n");
             SGVec3d velocity = SGVec3d::zeros();
             if ( _velocity[0] || _velocity[1] || _velocity[2] ) {
                 velocity = hlOr.backTransform(_velocity*SG_FEET_TO_METER);
-            }
 
-            if ( _bad_doppler ) {
-                velocity *= 100.0f;
+                if ( d->_bad_doppler ) {
+                    float fact = 100.0f;
+                    float mag = length( velocity );
+
+                    if (mag > _sound_velocity) {
+                        fact *= _sound_velocity / mag;
+                    }
+                    alDopplerFactor(fact);
+                }
             }
 
             alListenerfv( AL_VELOCITY, toVec3f(velocity).data() );
-            // alDopplerVelocity(340.3);	// TODO: altitude dependent
+//          alDopplerVelocity(_sound_velocity);
             testForError("update");
             _changed = false;
         }
@@ -448,7 +459,7 @@ bool SGSoundMgr::exists( const std::string &refname ) {
 
 
 // return a pointer to the SGSampleGroup if the specified sound exists
-// in the sound manager system, otherwise return NULL
+// in the sound manager system, otherwise return nullptr
 SGSampleGroup *SGSoundMgr::find( const std::string &refname, bool create ) {
     auto sample_grp_it = d->_sample_groups.find( refname );
     if ( sample_grp_it == d->_sample_groups.end() ) {
@@ -459,7 +470,7 @@ SGSampleGroup *SGSoundMgr::find( const std::string &refname, bool create ) {
             return sgrp;
         }
         else 
-            return NULL;
+            return nullptr;
     }
 
     return sample_grp_it->second;
@@ -525,7 +536,7 @@ unsigned int SGSoundMgr::request_buffer(SGSoundSample *sample)
     if ( !sample->is_valid_buffer() ) {
         // sample was not yet loaded or removed again
         std::string sample_name = sample->get_sample_name();
-        void *sample_data = NULL;
+        void* sample_data = nullptr;
 
         // see if the sample name is already cached
         auto buffer_it = d->_buffers.find( sample_name );
@@ -784,7 +795,7 @@ void SGSoundMgr::update_sample_config( SGSoundSample *sample, SGVec3d& position,
 
 
 bool SGSoundMgr::load( const std::string &samplepath,
-                       void **dbuf,
+                       void** dbuf,
                        int *fmt,
                        size_t *sz,
                        int *frq,
@@ -797,18 +808,16 @@ bool SGSoundMgr::load( const std::string &samplepath,
     unsigned int blocksz;
     ALsizei size;
     ALsizei freq;
-    ALvoid *data;
-
     ALfloat freqf;
 
-    data = simgear::loadWAVFromFile(samplepath, format, size, freqf, blocksz);
+    auto data = simgear::loadWAVFromFile(samplepath, format, size, freqf, blocksz);
     freq = (ALsizei)freqf;
-    if (data == NULL) {
+    if (data == nullptr) {
         throw sg_io_exception("Failed to load wav file", sg_location(samplepath));
     }
 
     if (format == AL_FORMAT_STEREO8 || format == AL_FORMAT_STEREO16) {
-        free(data);
+         free(data);
         throw sg_io_exception("Warning: STEREO files are not supported for 3D audio effects: " + samplepath);
     }
 
@@ -827,10 +836,10 @@ vector<std::string> SGSoundMgr::get_available_devices()
 #ifdef ENABLE_SOUND
     const ALCchar *s;
 
-    if (alcIsExtensionPresent(NULL, "ALC_enumerate_all_EXT") == AL_TRUE) {
-        s = alcGetString(NULL, ALC_ALL_DEVICES_SPECIFIER);
+    if (alcIsExtensionPresent(nullptr, "ALC_enumerate_all_EXT") == AL_TRUE) {
+        s = alcGetString(nullptr, ALC_ALL_DEVICES_SPECIFIER);
     } else {
-        s = alcGetString(NULL, ALC_DEVICE_SPECIFIER);
+        s = alcGetString(nullptr, ALC_DEVICE_SPECIFIER);
     }
 
     if (s) {
@@ -851,7 +860,7 @@ vector<std::string> SGSoundMgr::get_available_devices()
 
 bool SGSoundMgr::testForError(void *p, std::string s)
 {
-   if (p == NULL) {
+   if (p == nullptr) {
       SG_LOG( SG_SOUND, SG_ALERT, "Error: " << s);
       return true;
    }
@@ -889,7 +898,7 @@ bool SGSoundMgr::testForALCError(std::string s)
 
 bool SGSoundMgr::is_working() const 
 {
-    return (d->_device != NULL);
+    return (d->_device != nullptr);
 }
 
 const SGQuatd& SGSoundMgr::get_orientation() const
